'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _ScrollStore = require('./store/ScrollStore.js');

var _ScrollStore2 = _interopRequireDefault(_ScrollStore);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var SCROLLING_TIME_CONSTANT = 525;

var VerticalScroller = function () {
  function VerticalScroller(scrollContainer, scrollCallback) {
    _classCallCheck(this, VerticalScroller);

    this.scrollContainer = scrollContainer;
    this.scrollCallback = scrollCallback;

    // init variables
    this.timestamp = 0;
    this.minOffset = 0;
    this.maxOffset = Number.MAX_SAFE_INTEGER;
    this.frame = 0;
    this.velocity = 0;
    this.amplitude = 0;
    this.pressed = 0;
    this.reference = 0;
    this.offset = 0;
    this.target = 0;
    this.touchPositions = [];
    this.bindEvents();
  }

  _createClass(VerticalScroller, [{
    key: 'bindEvents',
    value: function bindEvents() {
      // rebinding the event handler to this
      this.tap = this.tap.bind(this);
      this.drag = this.drag.bind(this);
      this.release = this.release.bind(this);

      if (this.scrollContainer && this.scrollContainer.addEventListener) {
        this.scrollContainer.addEventListener('touchstart', this.tap);
        this.scrollContainer.addEventListener('touchmove', this.drag);
        this.scrollContainer.addEventListener('touchend', this.release);
        this.scrollContainer.addEventListener('mousedown', this.tap);
        this.scrollContainer.addEventListener('mousemove', this.drag);
        this.scrollContainer.addEventListener('mouseup', this.release);
      }
    }
  }, {
    key: 'destroy',
    value: function destroy() {
      if (this.scrollContainer && this.scrollContainer.addEventListener) {
        this.scrollContainer.removeEventListener('touchstart', this.tap);
        this.scrollContainer.removeEventListener('touchmove', this.drag);
        this.scrollContainer.removeEventListener('touchend', this.release);
        this.scrollContainer.removeEventListener('mousedown', this.tap);
        this.scrollContainer.removeEventListener('mousemove', this.drag);
        this.scrollContainer.removeEventListener('mouseup', this.release);
      }
      this.removeAllListener();
      this.scrollContainer = null;
      this.scrollCallback = null;
    }
    // check what type of event we received and get the right position out of it

  }, {
    key: 'getYPosition',
    value: function getYPosition(e) {
      // touch event
      if (e.targetTouches && e.targetTouches.length >= 1) {
        return e.targetTouches[0].clientY;
      }
      // mouse event
      return e.clientY;
    }
  }, {
    key: 'scroll',
    value: function scroll(position) {
      this.offset = position;
      this.scrollCallback(position);
      _ScrollStore2.default.set('type', 'on-scroll');
      // the direction will be set by the scroll start
      _ScrollStore2.default.emitChange('on-scroll');
    }
  }, {
    key: 'autoScroll',
    value: function autoScroll() {
      var elapsed = void 0,
          delta = void 0,
          newOffset = void 0;
      if (this.amplitude) {
        elapsed = Date.now() - this.timestamp;
        delta = this.amplitude * Math.exp(-elapsed / SCROLLING_TIME_CONSTANT);
        newOffset = this.target - delta;
        if (newOffset < this.minOffset) {
          if (this.target - delta >= this.minOffset - 2) {
            this.scroll(this.minOffset);
            return;
          }
          this.bounce(true);
        } else if (newOffset > this.maxOffset) {
          if (this.target - delta <= this.maxOffset + 2) {
            this.scroll(this.maxOffset);
            return;
          }
          this.bounce(false);
        } else if (delta > 2 || delta < -2) {
          this.scroll(this.target - delta);
          requestAnimationFrame(this.autoScroll.bind(this));
        } else {
          this.scroll(this.target);
          this.dispatchScrollEndEvent();
        }
      }
    }
  }, {
    key: 'dispatchScrollEndEvent',
    value: function dispatchScrollEndEvent() {
      _ScrollStore2.default.set('type', 'scroll-end');
      _ScrollStore2.default.emitChange('scroll-end');
    }
  }, {
    key: 'dispatchScrollStart',
    value: function dispatchScrollStart(direction) {
      _ScrollStore2.default.set('type', 'scroll-start');
      _ScrollStore2.default.set('direction', direction);
      _ScrollStore2.default.emitChange('scroll-start');
    }
  }, {
    key: 'bounce',
    value: function bounce(top) {
      var finalDestination = top ? this.minOffset : this.maxOffset,
          isBouncingBack = top && this.amplitude > 0 || !top && this.amplitude < 0;

      if (this.amplitude === 0) {
        return;
      }
      var elapsed = Date.now() - this.timestamp;
      var delta = this.amplitude * Math.exp(-elapsed / (this.target === finalDestination ? 125 : SCROLLING_TIME_CONSTANT));
      if (isBouncingBack && Math.abs(delta) < 2) {
        this.scroll(top ? this.minOffset : this.maxOffset);
        this.dispatchScrollEndEvent();
        return;
      }
      this.scroll(this.target - delta);

      if (isBouncingBack) {
        if (this.target !== finalDestination) {
          this.target = finalDestination;
          this.amplitude = this.target - this.offset;
          this.timestamp = new Date();
        }
      } else {
        this.target = finalDestination - (finalDestination - this.target) * 0.1;
        this.amplitude = this.target - this.offset;
      }

      requestAnimationFrame(function () {
        this.bounce(top);
      }.bind(this));
    }
  }, {
    key: 'tap',
    value: function tap(e) {
      this.pressed = true;
      this.reference = this.getYPosition(e);

      this.velocity = this.amplitude = 0;
      this.frame = this.offset;
      this.timestamp = Date.now();
      this.recordTouches(e);
      e.preventDefault && e.preventDefault();
      e.stopPropagation && e.stopPropagation();
    }
  }, {
    key: 'drag',
    value: function drag(e) {
      var y = void 0,
          delta = void 0,
          scaleFactor = this.offset < this.minOffset || this.offset > this.maxOffset ? 0.5 : 1;
      if (this.pressed) {
        this.recordTouches(e);
        y = this.getYPosition(e);
        delta = this.reference - y;
        if (delta > 2 || delta < -2) {
          this.reference = y;
          this.scroll(this.offset + delta * scaleFactor);
        }
      }
      e.preventDefault && e.preventDefault();
      e.stopPropagation && e.stopPropagation();
    }
  }, {
    key: 'recordTouches',
    value: function recordTouches(e) {
      var touches = e.touches || [{ pageX: e.pageX, pageY: e.pageY }],
          timestamp = e.timeStamp,
          currentTouchTop = touches[0].pageY;

      if (touches.length === 2) {
        currentTouchTop = Math.abs(touches[0].pageY + touches[1].pageY) / 2;
      }

      this.touchPositions.push({ offset: currentTouchTop, timestamp: timestamp });
      if (this.touchPositions.length > 60) {
        this.touchPositions.splice(0, 30);
      }
    }
  }, {
    key: 'release',
    value: function release(e) {
      this.pressed = false;
      var endPos = this.touchPositions.length - 1;
      var startPos = endPos - 1;
      // Move pointer to position measured 100ms ago
      for (var i = endPos - 1; i > 0 && this.touchPositions[i].timestamp > this.touchPositions[endPos].timestamp - 100; i -= 1) {
        startPos = i;
      }
      var elapsed = this.touchPositions[endPos].timestamp - (this.touchPositions[startPos] && this.touchPositions[startPos].timestamp) || 0;
      var delta = this.touchPositions[endPos].offset - (this.touchPositions[startPos] && this.touchPositions[startPos].offset) || 0;
      var v = -1000 * delta / (1 + elapsed);

      this.dispatchScrollStart(delta > 0 ? 1 : -1);

      this.velocity = 0.8 * v + 0.2 * this.velocity;
      this.amplitude = 1.0 * this.velocity;
      this.target = Math.round(this.offset + this.amplitude);
      this.timestamp = Date.now();
      requestAnimationFrame(this.autoScroll.bind(this));

      e.preventDefault && e.preventDefault();
      e.stopPropagation && e.stopPropagation();
    }
  }, {
    key: 'scrollTo',
    value: function scrollTo(y, animate, duration) {
      var maxAnimateDelta = duration || 4000;
      if (animate) {
        if (y - this.offset > maxAnimateDelta) {
          this.offset = y - maxAnimateDelta;
        } else if (this.offset - y > maxAnimateDelta) {
          this.offset = y + maxAnimateDelta;
        }

        this.amplitude = y - this.offset;
        this.target = y;
        this.timestamp = Date.now();
        requestAnimationFrame(this.autoScroll.bind(this));
      } else {
        this.amplitude = 0;
        this.scroll(y);
      }
    }
  }, {
    key: 'changeScrollPosition',
    value: function changeScrollPosition(y) {
      this.scroll(y);
    }
  }, {
    key: 'setDimensions',
    value: function setDimensions(min, max) {
      this.minOffset = min;
      this.maxOffset = max;
    }
  }, {
    key: 'addEventListener',
    value: function addEventListener(type, listener) {
      _ScrollStore2.default.on(type, listener);
    }
  }, {
    key: 'removeEventListener',
    value: function removeEventListener(type, listener) {
      _ScrollStore2.default.removeListener(type, listener);
    }
  }, {
    key: 'removeAllListener',
    value: function removeAllListener() {
      _ScrollStore2.default.removeAllListeners('scroll-end');
      _ScrollStore2.default.removeAllListeners('scroll-start');
      _ScrollStore2.default.removeAllListeners('on-scroll');
    }
  }]);

  return VerticalScroller;
}();

exports.default = VerticalScroller;