'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _VerticalScroller = require('./VerticalScroller.js');

var _VerticalScroller2 = _interopRequireDefault(_VerticalScroller);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var VirtualScroll = function () {
  /**
   * constructor - VirtualScroll
   *
   * @param  {object} config
   * {
   *  // DOMnode the list will be renderd inside
   *  root: document.getElementsByClassName('NAME_THE_CONTAINER_WHATEVER_YOU_LIKE')[0],
   *  // ARRAY containg all the objects to be displayed inside the list
   *  source: listSource,
   *  //FUNCTION to be called while creating a single item
   *    itemNodes empty object provided by the list logic
   *    itemContainer   item container provided by the list logic
   *  createItemFn: (itemNodes, itemContainer) =>{},
   *
   * //FUNCTION to be called while updating a single item
   *   //itemNodes       object previously filled by createItemFn()
   *  //itemContainer   item container provided by the list logic
   *  //itemData        object inside listSource array at the current index
   * updateItemFn: (itemNodes, itemContainer, itemData) => {},
   * }
   */
  function VirtualScroll(config) {
    _classCallCheck(this, VirtualScroll);

    this.config = {};
    this.info = { scrollTop: 0, direction: 1, height: 1, isScrolling: false };
    this.config.root = config && config.root ? config.root : document.createElement('div');
    this.config.source = config && config.source ? config.source : [];
    this.config.createItemFn = config && config.createItemFn ? config.createItemFn : null;
    this.config.updateItemFn = config && config.updateItemFn ? config.updateItemFn : null;
    this.rootElement = null;
    // setup the container

    this.config.scroller = config && config.scroller ? config.scroller : new _VerticalScroller2.default(this.config.root, this.scroll.bind(this));

    // TODO try to get dynamic itemHeight
    this.totalRows = this.config.source.length;
    this.itemHeight = config && config.itemHeight ? config.itemHeight : 50;
    this.visibleItemsCount = Math.ceil(this.config.root.offsetHeight / this.itemHeight);
    this.cachedItemsLen = this.visibleItemsCount * 3;
    this.setupContainer();
    this.renderChunk(this.rootElement, 0);
    this.scrollerElement = this.createScroller(this.totalRows * this.itemHeight);
    this.info.height = this.totalRows * this.itemHeight; // estimated for now
    var bottom = this.info.height - (this.visibleItemsCount - 1) * this.itemHeight;
    this.config.scroller.setDimensions(0, bottom);
    this.rootElement.appendChild(this.scrollerElement);
    this.lastRepaintY = 0;
    this.maxBuffer = this.visibleItemsCount * this.itemHeight;
    this.lastScrolled = 0;
    this.rmNodeInterval = setInterval(this.remoDirtyNodes.bind(this), 300);
    this.bindEvents();
  }

  _createClass(VirtualScroll, [{
    key: 'remoDirtyNodes',
    value: function remoDirtyNodes() {
      if (Date.now() - this.lastScrolled > 100) {
        var badNodes = this.rootElement.querySelectorAll('[data-rm="1"]');
        for (var i = 0, l = badNodes.length; i < l; i++) {
          this.rootElement.removeChild(badNodes[i]);
        }
      }
    }
  }, {
    key: 'bindEvents',
    value: function bindEvents() {
      this.config.scroller.addEventListener('scroll-start', this.onScrollBegin.bind(this));
      this.config.scroller.addEventListener('scroll-end', this.onScrollEnd.bind(this));
    }
  }, {
    key: 'onScrollBegin',
    value: function onScrollBegin(data) {
      this.info.isScrolling = true;
      this.info.direction = data.direction;
    }
  }, {
    key: 'onScrollEnd',
    value: function onScrollEnd() {
      this.info.isScrolling = false;
    }
  }, {
    key: 'setupContainer',
    value: function setupContainer() {
      this.rootElement = document.createElement('div');
      this.rootElement.style.width = this.config.root.clientWidth + 'px';
      this.rootElement.style.height = this.config.root.clientHeight + 'px';

      this.config.root.style.overflow = 'hidden';
      this.config.root.style.position = 'relative';
      this.config.root.style.padding = 0;
      this.config.root.style.border = '1px solid black';

      this.config.root.appendChild(this.rootElement);
    }
  }, {
    key: 'createScroller',
    value: function createScroller(h) {
      var scroller = document.createElement('div');
      scroller.style.opacity = 0;
      scroller.style.position = 'absolute';
      scroller.style.top = 0;
      scroller.style.left = 0;
      scroller.style.width = '1px';
      scroller.style.height = h + 'px';
      return scroller;
    }
  }, {
    key: 'renderChunk',
    value: function renderChunk(node, from) {
      var finalItem = from + this.cachedItemsLen;
      if (finalItem > this.totalRows) {
        finalItem = this.totalRows;
      }
      // Append all the new rows in a document fragment that we will later append to
      // the parent node
      var fragment = document.createDocumentFragment();
      for (var i = from; i < finalItem; i++) {
        this.createRow(i, fragment);
      }

      // Hide and mark obsolete nodes for deletion.
      for (var j = 1, l = node.childNodes.length; j < l; j++) {
        if (this.scrollerElement !== node.childNodes[j]) {
          node.childNodes[j].style.display = 'none';
          node.childNodes[j].setAttribute('data-rm', '1');
        }
      }
      node.appendChild(fragment);
    }
  }, {
    key: 'createRow',
    value: function createRow(index, container) {
      var item = this.config.source[index];
      if (this.config.createItemFn) {
        var emptyObject = {};
        this.config.createItemFn(emptyObject, container);
        this.config.updateItemFn(emptyObject, container, item);
        var nodeItem = container.lastChild;
        nodeItem.classList.add('vrow');
        nodeItem.style.position = 'absolute';
        nodeItem.style.top = index * this.itemHeight + 'px';
      }
    }
  }, {
    key: 'scroll',
    value: function scroll(position) {
      //Update top of the root element
      // translation is faster than chaning top
      // for more info: http://www.paulirish.com/2012/why-moving-elements-with-translate-is-better-than-posabs-topleft/
      this.info.scrollTop = -position;

      // translateZ(0) is to trick the browser to use the GPU for the animation
      // so we end up with smoother animation
      var t = 'translateY(' + -position + 'px) translateZ(0)';
      var s = this.rootElement.style;
      s['transform'] = t;
      s['webkitTransform'] = t;
      s['mozTransform'] = t;
      s['msTransform'] = t;

      if (!this.lastRepaintY || Math.abs(position - this.lastRepaintY) > this.maxBuffer) {
        var first = parseInt(position / this.itemHeight) - this.visibleItemsCount;
        this.renderChunk(this.rootElement, first < 0 ? 0 : first);
        this.lastRepaintY = position;
      }
      this.lastScrolled = Date.now();
    }
    /*Public API*/

  }, {
    key: 'refresh',
    value: function refresh() {
      this.setupContainer();
      this.renderChunk(this.rootElement, 0);
      var bottom = this.itemHeight * this.totalRows - (this.visibleItemsCount - 1) * this.itemHeight;
      this.config.scroller.setDimensions(0, bottom);
    }
  }, {
    key: 'setSource',
    value: function setSource(listSource) {
      this.config.source = listSource;
      this.refresh();
    }
  }, {
    key: 'destroy',
    value: function destroy() {
      this.config.scroller.destroy();
      clearInterval(this.rmNodeInterval);
    }
  }, {
    key: 'remove',
    value: function remove() {
      this.destroy();
      this.config.root.removeChild(this.rootElement);
    }
  }, {
    key: 'addEventListener',
    value: function addEventListener(event, callback) {
      this.config.scroller.addEventListener(event, callback);
    }
  }, {
    key: 'removeEventListener',
    value: function removeEventListener(event, callback) {
      this.config.scroller.removeEventListener(event, callback);
    }
  }, {
    key: 'scrollTop',
    value: function scrollTop(duration) {
      this.scrollTo(0, duration);
      this.info.direction = -1;
    }
  }, {
    key: 'scrollBottom',
    value: function scrollBottom(duration) {
      var bottom = this.itemHeight * this.totalRows - (this.visibleItemsCount - 1) * this.itemHeight;
      this.scrollTo(bottom, duration);
      this.info.direction = 1;
      this.info.scrollTop = -bottom;
    }
  }, {
    key: 'scrollTo',
    value: function scrollTo(position, duration) {
      this.info.direction = position > this.info.scrollTop ? -1 : 1;
      var animate = duration && duration !== 0;
      this.config.scroller.dispatchScrollStart(this.info.direction);
      this.config.scroller.scrollTo(position, animate, duration);
    }
  }]);

  return VirtualScroll;
}();

exports.default = VirtualScroll;